package app.revanced.patches.kakaotalk.versioninfo

import app.revanced.patcher.extensions.InstructionExtensions.instructions
import app.revanced.patcher.extensions.InstructionExtensions.replaceInstruction
import app.revanced.patcher.patch.Package
import app.revanced.patcher.patch.bytecodePatch
import app.revanced.patches.kakaotalk.versioninfo.fingerprints.versionInfoFingerprint
import com.android.tools.smali.dexlib2.Opcode
import com.android.tools.smali.dexlib2.builder.instruction.BuilderInstruction21c
import com.android.tools.smali.dexlib2.iface.reference.StringReference
import com.android.tools.smali.dexlib2.immutable.reference.ImmutableStringReference

@Suppress("unused")
val versionInfoPatch = bytecodePatch(
    name = "Version info patch",
    description = "Patches the version info to include '(ReVanced)' in the version string.",
) {
    compatibleWith(Package("com.kakao.talk", setOf("25.3.5")))

    execute {
        val inst = versionInfoFingerprint.method.instructions
            .filterIsInstance<BuilderInstruction21c>()
            .first {
                it.opcode == Opcode.CONST_STRING
            }

        val versionString = (inst.reference as StringReference).string

        versionInfoFingerprint.method
            .replaceInstruction(
                inst.location.index,
                BuilderInstruction21c(
                    Opcode.CONST_STRING,
                    inst.registerA,
                    ImmutableStringReference(
                        "$versionString (ReVanced)"
                    )
                )
            )
    }
}